
/**
 * @file Box.cpp
 * @brief Implementation of class Box
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include "Box.h"

using namespace std;

Box::Box(teOrientation o,
         int border,
         int w,
         int h,
         teAlignment wa,
         teAlignment ha)
    : Container(border, w, h, wa, ha),
      _o(o)
{
    if (_w < 2 * _border)
    {
       _mw = 2 * border;
    }
    else
    {
       _mw = _w;
    }

    if (_h < 2* _border)
    {
       _mh = 2 * _border;
    }
    else
    {
       _mh = _h;
    }
}

Box::~Box()
{
    for (list<Sizeable*>::iterator k = _children.begin();
         k != _children.end();
         ++k)
    {
       delete *k;
    }
}

unsigned int Box::getMinWidth()
{
    return _mw;
}

unsigned int Box::getMinHeight()
{
    return _mh;
}

void Box::addChild(Sizeable *s)
{
    _children.push_back(s);
    if (_o == HORIZONTAL)
    {
       _mh = (_mh < s->getMinHeight()) ? s->getMinHeight() : _mh;
       _mw += s->getMinWidth();
       if (_children.size() != 1)
       {
          _mw += _border;
       }
    }
    else
    {
       _mw = (_mw < s->getMinWidth()) ? s->getMinWidth() : _mw;
       _mh += s->getMinHeight();
       if (_children.size() != 1)
       {
          _mh += _border;
       }
    }
}

void Box::adjust(int x, int y, int w, int h)
{
    _compute(HORIZONTAL, x, w);
    _compute(VERTICAL, y, h);

    int fixed = 0;
    int coeff = 0;

    for (list<Sizeable*>::iterator k = _children.begin();
         k != _children.end();
         ++k)
    {
       int d = (*k)->getDim(_o);

       if (d < 0 )
       {
          coeff += -d;
       }
       else
       {
          fixed += d;
       }
    }

    fixed += (_children.size() + 1) * _border;

    int *widths = new int[_children.size()];
    int k;
    for (k = 0; k<(int)_children.size(); ++k) widths[k] = -1;

    k = 0;
    int took = 0;
    int ncoeff = 0;

    list<Sizeable*>::iterator pos = _children.begin();
    while (pos != _children.end())
    {
       if ((*pos)->getDim(_o) > 0)
       {
          widths[k] = (*pos)->getDim(_o);
       }
       else
       {
          int d = - (*pos)->getDim(_o) * (((_o == HORIZONTAL) ? w : h) - fixed) / coeff;
          if (d < (int)(*pos)->getMinDim(_o))
          {
             widths[k] = (*pos)->getMinDim(_o);
             took += (*pos)->getMinDim(_o);
          }
          else
          {
             widths[k] = (*pos)->getDim(_o);
             ncoeff += (-widths[k]);
          }
       }

       ++k;
       ++pos;
    }

    fixed += took;
    for (k = 0; k<(int)_children.size(); ++k)
    {
       if (widths[k] < 0)
       {
          widths[k] = (((_o == HORIZONTAL) ? w : h) - fixed) * (-widths[k]) / ncoeff;
       }
    }

    x += _border;
    y += _border;

    k = 0;
    for (pos = _children.begin();
         pos != _children.end();
         ++pos, ++k)
    {
       if (_o == HORIZONTAL)
       {
          (*pos)->adjust(x, y, widths[k], h - 2 * _border);
          x += widths[k] + _border;
       }
       else
       {
          (*pos)->adjust(x, y, w - 2 * _border, widths[k]);
          y += widths[k] + _border;
       }
    }

    delete[] widths;
}

static const char* _rcsid_Box __attribute__((unused)) = "$Id$";
