
/**
 * @file ControlRC.cpp
 * @brief Implementation of class ControlRC
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include <sstream>

#include "ControlRC.h"
#include "IDGen.h"

using namespace std;

map<string, pair<string, string> > ControlRC::_classes;

void ControlRC::_initClasses()
{
    _classes.insert(make_pair("auto3",
                              make_pair("AUTO3STATE",
                                        "")));
    _classes.insert(make_pair("autocheck",
                              make_pair("AUTOCHECKBOX",
                                        "")));
    _classes.insert(make_pair("autoradio",
                              make_pair("AUTORADIOBUTTON",
                                        "")));
    _classes.insert(make_pair("checkbox",
                              make_pair("CHECKBOX",
                                        "")));
    _classes.insert(make_pair("combobox",
                              make_pair("COMBOBOX",
                                        "")));
    _classes.insert(make_pair("ctext",
                              make_pair("CTEXT",
                                        "")));
    _classes.insert(make_pair("defpush",
                              make_pair("DEFPUSHBUTTON",
                                        "")));
    _classes.insert(make_pair("edit",
                              make_pair("EDITTEXT",
                                        "")));
    _classes.insert(make_pair("icon",
                              make_pair("ICON",
                                        "")));
    _classes.insert(make_pair("listbox",
                              make_pair("LISTBOX",
                                        "")));
    _classes.insert(make_pair("ltext",
                              make_pair("LTEXT",
                                        "")));
    _classes.insert(make_pair("pushbox",
                              make_pair("PUSHBOX",
                                        "")));
    _classes.insert(make_pair("push",
                              make_pair("PUSHBUTTON",
                                        "")));
    _classes.insert(make_pair("radio",
                              make_pair("RADIOBUTTON",
                                        "")));
    _classes.insert(make_pair("rtext",
                              make_pair("RTEXT",
                                        "")));
    _classes.insert(make_pair("scroll",
                              make_pair("SCROLLBAR",
                                        "")));
    _classes.insert(make_pair("state3",
                              make_pair("STATE3",
                                        "")));
    _classes.insert(make_pair("group",
                              make_pair("GROUPBOX",
                                        "")));
}

bool ControlRC::findInt(const map<string, string>& a,
                        const string& n,
                        int& r)
{
    map<string, string>::const_iterator pos = a.find(n);
    if (pos != a.end())
    {
       r = atoi(pos->second.c_str());
       return true;
    }

    return false;
}

bool ControlRC::findString(const map<string, string>& a,
                           const string& n,
                           string& r)
{
    map<string, string>::const_iterator pos = a.find(n);
    if (pos != a.end())
    {
       r = pos->second;
       return true;
    }

    return false;
}

ControlRC::ControlRC(ostream& strm,
                     const string& cl,
                     const map<string, string>& attrs) throw (string)
    : Sizeable(),
      _str(strm),
      _class(cl)
{
    findInt(attrs, "w", _w);
    findInt(attrs, "h", _h);
    findInt(attrs, "rh", _rh);

    string str;
    if (findString(attrs, "walign", str))
    {
       if (str == "left")
       {
          _wa = TOPLEFT;
       }
       else if (str == "right")
       {
          _wa = BOTTOMRIGHT;
       }
    }

    if (findString(attrs, "halign", str))
    {
       if (str == "top")
       {
          _ha = TOPLEFT;
       }
       else if (str == "bottom")
       {
          _ha = BOTTOMRIGHT;
       }
    }

    findString(attrs, "style", _style);
    if (!findString(attrs, "id", _id))
    {
       ostringstream s;
       s << IDGen::getNewID();
       _id = s.str();
    }

    if (_class == "control")
    {
       if (!findString(attrs, "class", _class))
       {
          throw string("Cannot create a control without class.");
       }
    }
}

ControlRC::~ControlRC()
{
}

void ControlRC::adjust(int x, int y, int w, int h)
{
    _compute(HORIZONTAL, x, w);
    _compute(VERTICAL, y, h);

    if (_classes.size() == 0)
    {
       _initClasses();
    }

    map<string, pair<string, string> >::iterator pos;
    pos = _classes.find(_class);
    if (pos == _classes.end())
    {
       _str << "   CONTROL \"" << _text << "\", " << _id << ", ";
       _str << "\"" << _class << "\"";
       _str << ", " << ((_style == "") ? "0" : _style) << ", ";
       _str << x << ", " << y << ", " << w << ", " << h;
    }
    else
    {
       _str << "   " << pos->second.first << " \"" << _text << "\", ";
       _str << _id << ", " << x << ", " << y << ", " << w << ", ";

       if (_rh > 0)
       {
          _str << _rh;
       }
       else
       {
          _str << h;
       }

       if ((_style != "") || (pos->second.second != ""))
       {
          string st = _style;
          if (pos->second.second != "")
          {
             if (st != "") st += " | ";
             st += pos->second.second;
          }
          _str << ", " << st;
       }
    }

    _str << endl;
}

void ControlRC::setText(const string& t)
{
    _text = t;
}

static const char* _rcsid_ControlRC __attribute__((unused)) = "$Id$";
