
/**
 * @file Parser.cpp
 * @brief Implementation of class Parser
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include <sstream>

#include "Parser.h"
#include "IDGen.h"
#include "Icon.h"
#include "Box.h"
#include "MenuItem.h"
#include "DialogRC.h"
#include "Container.h"
#include "ControlRC.h"
#include "ContainerControlRC.h"

using namespace std;

#define NONE            0
#define IN_RC           1
#define IN_ID           2
#define IN_ICON         3
#define IN_RAW          4
#define IN_MENU         5
#define IN_DIALOG       6
#define IN_STYLE        7
#define IN_CAPTION      8
#define IN_FONT         9
#define IN_CONTAINER    10
#define IN_CONTROL      11

static void findMisc(map<string, string> m,
                     int& border,
                     int& w,
                     int& h,
                     Sizeable::teAlignment& wa,
                     Sizeable::teAlignment& ha)
{
    border = 2;
    w = -1;
    h = -1;
    wa = Sizeable::CENTER;
    ha = Sizeable::CENTER;

    ControlRC::findInt(m, "border", border);
    ControlRC::findInt(m, "w", w);
    ControlRC::findInt(m, "h", h);

    string s;
    if (ControlRC::findString(m, "walign", s))
    {
       if (s == "left")
       {
          wa = Sizeable::TOPLEFT;
       }
       else if (s == "right")
       {
          wa = Sizeable::BOTTOMRIGHT;
       }
    }

    if (ControlRC::findString(m, "halign", s))
    {
       if (s == "top")
       {
          ha = Sizeable::TOPLEFT;
       }
       else if (s == "bottom")
       {
          ha = Sizeable::BOTTOMRIGHT;
       }
    }
}

Parser::Parser(ostream& rc,
               ostream& h,
               bool genh)
    : _rc(rc),
      _h(h),
      _genh(genh),
      _state(NONE)
{
}

Parser::~Parser()
{
    for (list<U_STACK>::iterator k = _ostack.begin();
         k != _ostack.end();
         ++k)
    {
       switch (k->type)
       {
          case TYPE_ID:
             delete k->value.id;
             break;
          case TYPE_ICON:
             delete k->value.icn;
             break;
          case TYPE_RAW:
             delete k->value.raw;
             break;
          case TYPE_MENU:
             delete k->value.menu;
             break;
          case TYPE_DIALOG:
             delete k->value.dlg;
             break;
          case TYPE_CONTAINER:
             delete k->value.ctn;
             break;
          case TYPE_CONTROL:
             delete k->value.ctrl;
             break;
          default:
             break;
       }
    }
}

void Parser::startElement(const string& name,
                          const map<string, string>& attrs)
{
    _text = "";

    // Cas particulier, on peut mettre du raw n'importe o
    // PS c'est faux
    if (name == "raw")
    {
       _stack.push_front(_state);
       _state = IN_RAW;
    }
    else
    {
       switch (_state)
       {
          case NONE:
             if (name != "rc")
             {
                cerr << "Misplaced " << name << " at top level" << endl;
                exit(1);
             }
             _stack.push_front(_state);
             _state = IN_RC;
             break;

          case IN_RC:
             // Accepted tags here: id, icon, menu, dialog
             if (name == "id")
             {
                U_STACK st;
                st.type = TYPE_ID;
                st.value.id = new IDGen();
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_ID;
                break;
             }
             else if (name == "icon")
             {
                map<string, string>::const_iterator pos = attrs.find("file");
                if (pos == attrs.end())
                {
                   cerr << "Icon must have a filename" << endl;
                   exit(1);
                }

                U_STACK st;
                st.type = TYPE_ICON;
                st.value.icn = new Icon("", pos->second.c_str());
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_ICON;
             }
             else if (name == "menu")
             {
                map<string, string>::const_iterator pos = attrs.find("name");
                if (pos == attrs.end())
                {
                   cerr << "Menu must have a name" << endl;
                   exit(1);
                }

                U_STACK st;
                st.type = TYPE_MENU;
                st.value.menu = new MenuItem(pos->second.c_str());
                _ostack.push_front(st);
                _stack.push_front(_state);
                _state = IN_MENU;
             }
             else if (name == "dialog")
             {
                map<string, string>::const_iterator pos = attrs.find("name");
                if (pos == attrs.end())
                {
                   cerr << "Dialog must have a name" << endl;
                   exit(1);
                }
                string nm = pos->second;

                int w = -1;
                pos = attrs.find("w");
                if (pos != attrs.end())
                {
                   w = atoi(pos->second.c_str());
                }

                int h = -1;
                pos = attrs.find("h");
                if (pos != attrs.end())
                {
                   h = atoi(pos->second.c_str());
                }

                U_STACK st;
                st.type = TYPE_DIALOG;
                st.value.dlg = new DialogRC(_rc, nm, w, h);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_DIALOG;
             }
             break;

          case IN_MENU:
          {
             if (name != "menu")
             {
                cerr << "misplaced " << name << endl;
                exit(1);
             }

             map<string, string>::const_iterator pos = attrs.find("name");
             if (pos == attrs.end())
             {
                cerr << "Menu must have a name" << endl;
                exit(1);
             }

             U_STACK st;
             st.type = TYPE_MENU;
             st.value.menu = new MenuItem(pos->second.c_str());

             pos = attrs.find("id");
             if (pos != attrs.end())
             {
                st.value.menu->setID(pos->second.c_str());
             }

             _ostack.push_front(st);
             _stack.push_front(_state);
             _state = IN_MENU;
          }
          break;

          case IN_DIALOG:
          {
             if (name == "style")
             {
                _stack.push_front(_state);
                _state = IN_STYLE;
             }
             else if (name == "caption")
             {
                _stack.push_front(_state);
                _state = IN_CAPTION;
             }
             else if (name == "font")
             {
                int size = 8;
                map<string, string>::const_iterator pos = attrs.find("size");
                if (pos != attrs.end())
                {
                   size = atoi(pos->second.c_str());
                }
                // bidouillage en interne :)
                U_STACK st;
                st.type = TYPE_INT;
                st.value.ival = size;
                _ostack.push_front(st);
                _stack.push_front(_state);
                _state = IN_FONT;
             }
             else if (name == "group")
             {
                int w, h, border, hborder = 16;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);
                ControlRC::findInt(attrs, "hborder", hborder);
                ControlRC *ctrl = new ControlRC(_rc, name, attrs);

                string s;
                if (ControlRC::findString(attrs, "text", s))
                {
                   ctrl->setText(s);
                }

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new ContainerControlRC(ctrl,
                                                      hborder,
                                                      border,
                                                      w, h,
                                                      wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "vbox")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new Box(Sizeable::VERTICAL,
                                       border,
                                       w, h,
                                       wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "hbox")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new Box(Sizeable::HORIZONTAL,
                                       border,
                                       w, h,
                                       wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "simple")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new SimpleContainer(border,
                                                   w, h,
                                                   wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else
             {
                // contrle
                U_STACK st;
                st.type = TYPE_CONTROL;
                st.value.ctrl = new ControlRC(_rc,
                                              name,
                                              attrs);

                _ostack.push_front(st);
                _stack.push_front(_state);
                _state = IN_CONTROL;
             }
          }
          break;

          case IN_CONTAINER:
          {
             if (name == "group")
             {
                int w, h, border, hborder = 16;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);
                ControlRC::findInt(attrs, "hborder", hborder);
                ControlRC *ctrl = new ControlRC(_rc, name, attrs);

                string s;
                if (ControlRC::findString(attrs, "text", s))
                {
                   ctrl->setText(s);
                }

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new ContainerControlRC(ctrl,
                                                      hborder,
                                                      border,
                                                      w, h,
                                                      wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "vbox")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new Box(Sizeable::VERTICAL,
                                       border,
                                       w, h,
                                       wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "hbox")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new Box(Sizeable::HORIZONTAL,
                                       border,
                                       w, h,
                                       wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else if (name == "simple")
             {
                int w, h, border;
                Sizeable::teAlignment wa, ha;
                findMisc(attrs, border, w, h, wa, ha);

                U_STACK st;
                st.type = TYPE_CONTAINER;
                st.value.ctn = new SimpleContainer(border,
                                                   w, h,
                                                   wa, ha);
                _ostack.push_front(st);

                _stack.push_front(_state);
                _state = IN_CONTAINER;
             }
             else
             {
                // contrle
                U_STACK st;
                st.type = TYPE_CONTROL;
                st.value.ctrl = new ControlRC(_rc,
                                              name,
                                              attrs);

                _ostack.push_front(st);
                _stack.push_front(_state);
                _state = IN_CONTROL;
             }
          }
          break;
       }
    }
}

void Parser::endElement(const string& name)
{
    switch (_state)
    {
       case IN_RC:
          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
          break;

       case IN_ID:
       {
          if (_text == "")
          {
             cerr << "ID must have a name!" << endl;
             exit(1);
          }

          IDGen *g = _ostack.begin()->value.id;
          g->setName(_text.c_str());

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_ICON:
       {
          if (_text == "")
          {
             cerr << "Icon must have a name" << endl;
             exit(1);
          }

          Icon *i = _ostack.begin()->value.icn;
          i->setName(_text.c_str());

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_RAW:
       {
          U_STACK st;
          st.type = TYPE_RAW;
          st.value.raw = new string(_text);
          _ostack.push_front(st);

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_MENU:
       {
          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
          if (_state == IN_MENU)
          {
             // sous-menu
             U_STACK st = *(_ostack.begin());
             _ostack.erase(_ostack.begin());
             U_STACK p = *(_ostack.begin());
             p.value.menu->addChild(st.value.menu);
          }
       }
       break;

       case IN_DIALOG:
          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
          break;

       case IN_STYLE:
       {
          U_STACK st = *(_ostack.begin());
          st.value.dlg->setStyle(_text);

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_CAPTION:
       {
          U_STACK st = *(_ostack.begin());
          st.value.dlg->setCaption(_text);

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_FONT:
       {
          U_STACK st = *(_ostack.begin());
          _ostack.erase(_ostack.begin());

          U_STACK d = *(_ostack.begin());
          d.value.dlg->setFontSize(st.value.ival);
          d.value.dlg->setFont(_text);

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
       }
       break;

       case IN_CONTAINER:
          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
          switch (_state)
          {
             case IN_CONTAINER:
             {
                U_STACK st = *(_ostack.begin());
                _ostack.erase(_ostack.begin());
                U_STACK p = *(_ostack.begin());
                p.value.ctn->addChild(st.value.ctn);
             }
             break;

             case IN_DIALOG:
             {
                U_STACK st = *(_ostack.begin());
                _ostack.erase(_ostack.begin());
                U_STACK p = *(_ostack.begin());
                p.value.dlg->addChild(st.value.ctn);
             }
             break;

             default:
                break;
          }
          break;

       case IN_CONTROL:
       {
          U_STACK st = *(_ostack.begin());
          _ostack.erase(_ostack.begin());
          st.value.ctrl->setText(_text);

          _state = *(_stack.begin());
          _stack.erase(_stack.begin());
          U_STACK p = *(_ostack.begin());
          switch (_state)
          {
             case IN_CONTAINER:
                p.value.ctn->addChild(st.value.ctrl);
                break;
             case IN_DIALOG:
                p.value.dlg->addChild(st.value.ctrl);
                break;
          }
       }
       break;
    }
}

void Parser::text(const string& s)
{
    _text += s;
}

void Parser::parse(const string& s)
{
    SAXParser::parse(s);
    if (_state != NONE)
    {
       cerr << "Parse error..." << endl;
       exit(1);
    }

    for (list<U_STACK>::reverse_iterator k = _ostack.rbegin();
         k != _ostack.rend();
         ++k)
    {
       switch (k->type)
       {
          case TYPE_ID:
             if (_genh)
             {
                _h << "#define " << k->value.id->getName();
                _h << " " << k->value.id->getValue() << endl;
             }
             break;
          case TYPE_ICON:
             _rc << k->value.icn->getName() << " ICON DISCARDABLE \"";
             _rc << k->value.icn->getFile() << "\"" << endl;
             break;
          case TYPE_RAW:
             _rc << k->value.raw->c_str() << endl;
             break;
          case TYPE_MENU:
             _rc << endl;
             _rc << k->value.menu->getText() << " MENU" << endl;
             _rc << "BEGIN" << endl;
             k->value.menu->dumpChildren(_rc, 1);
             _rc << "END" << endl;
             break;
          case TYPE_DIALOG:
             // RC
             k->value.dlg->resize();
             // H
             if (_genh)
             {
                k->value.dlg->accept(this);
             }
             break;
          default:
             cerr << "Unrecognized type in stack!" << endl;
             exit(1);
       }
    }
}

void Parser::visit(DialogRC *v)
{
    _h << endl;
    _h << "class Resizer_" << v->getName() << " : public DialogRuntime" << endl;
    _h << "{" << endl;
    _h << "  public:" << endl;
    _h << "    Resizer_" << v->getName() << "(HWND hDlg)" << endl;
    _h << "        : DialogRuntime(hDlg, -1, -1)" << endl;
    _h << "    {" << endl;

    _h << "        RECT rect;" << endl;
    _h << "        memset(&rect, 0, sizeof(rect));" << endl;
    _h << "        RECT rborder;" << endl;
    _h << "        memset(&rborder, 0, sizeof(rborder));" << endl;
    _h << endl;

    v->getChild()->accept(this);

    _h << endl;
    _h << "        addChild(obj" << (void*)v->getChild() << ");" << endl;
    _h << "    }" << endl;
    _h << "    virtual ~Resizer_" << v->getName() << "() {};" << endl;
    _h << "};" << endl;
}

void Parser::visit(SimpleContainer *v)
{
    v->getChild()->accept(this);

    if ((v->getWidth() > 0) || (v->getHeight() > 0))
    {
       _h << "        rect.bottom = " << v->getHeight() << ";" << endl;
       _h << "        rect.right = " << v->getWidth() << ";" << endl;
       _h << "        MapDialogRect(hDlg, &rect);" << endl;
    }

    _h << "        rborder.bottom = " << v->getBorder() << ";" << endl;
    _h << "        MapDialogRect(hDlg, &rborder);" << endl;

    _h << "        SimpleContainer *obj" << (void*)v << " = ";
    _h << "new SimpleContainer(rborder.bottom, ";

    if (v->getWidth() > 0)
    {
       _h << "rect.right, ";
    }
    else
    {
       _h << v->getWidth() << ", ";
    }

    if (v->getHeight() > 0)
    {
       _h << "rect.bottom, ";
    }
    else
    {
       _h << v->getHeight() << ", ";
    }

    switch (v->getWAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ", ";
    switch (v->getHAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ");" << endl;
    _h << "        obj" << (void*)v << "->addChild(obj" << (void*)v->getChild() << ");" << endl;
}

void Parser::visit(Box *v)
{
    list<Sizeable*> c = v->getChildren();
    for (list<Sizeable*>::iterator k = c.begin();
         k != c.end();
         ++k)
    {
       (*k)->accept(this);
    }

    if ((v->getWidth() > 0) || (v->getHeight() > 0))
    {
       _h << "        rect.right = " << v->getWidth() << ";" << endl;
       _h << "        rect.bottom = " << v->getHeight() << ";" << endl;
       _h << "        MapDialogRect(hDlg, &rect);" << endl;
    }

    _h << "        rborder.bottom = " << v->getBorder() << ";" << endl;
    _h << "        MapDialogRect(hDlg, &rborder);" << endl;

    _h << "        Box *obj" << (void*)v << " = new Box(";
    switch (v->getOrientation())
    {
       case Sizeable::HORIZONTAL:
          _h << "HORIZONTAL";
          break;
       case Sizeable::VERTICAL:
          _h << "VERTICAL";
          break;
    }

    _h << ", rborder.bottom, ";

    if (v->getWidth() > 0)
    {
       _h << "rect.right, ";
    }
    else
    {
       _h << v->getWidth() << ", ";
    }

    if (v->getHeight() > 0)
    {
       _h << "rect.bottom, ";
    }
    else
    {
       _h << v->getHeight() << ", ";
    }


    switch (v->getWAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ", ";
    switch (v->getHAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ");" << endl;

    for (list<Sizeable*>::iterator k = c.begin();
         k != c.end();
         ++k)
    {
       _h << "        obj" << (void*)v << "->addChild(obj";
       _h << (void*)(*k) << ");" << endl;
    }
}

void Parser::visit(ControlRC *v)
{
    if ((v->getWidth() > 0) || (v->getHeight() > 0))
    {
       _h << "        rect.right = " << v->getWidth() << ";" << endl;
       _h << "        rect.bottom = " << v->getHeight() << ";" << endl;
       _h << "        MapDialogRect(hDlg, &rect);" << endl;
    }

    _h << "        ControlRuntime *obj" << (void*)v << " = ";
    _h << "new ControlRuntime(hDlg, " << v->getID() << ", ";

    if (v->getWidth() > 0)
    {
       _h << "rect.right, ";
    }
    else
    {
       _h << v->getWidth() << ", ";
    }

    if (v->getHeight() > 0)
    {
       _h << "rect.bottom, ";
    }
    else
    {
       _h << v->getHeight() << ", ";
    }

    switch (v->getWAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ", ";
    switch (v->getHAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ");" << endl;

    if (v->getRH() > 0)
    {
       _h << "        obj" << (void*)v << "->setRH(" << v->getRH() << ");" << endl;
    }
}

void Parser::visit(ContainerControlRC *v)
{
    v->getControl()->accept(this);
    v->getChild()->accept(this);

    if ((v->getWidth() > 0) || (v->getHeight() > 0))
    {
       _h << "        rect.right = " << v->getWidth() << ";" << endl;
       _h << "        rect.bottom = " << v->getHeight() << ";" << endl;
       _h << "        MapDialogRect(hDlg, &rect);" << endl;
    }

    _h << "        rborder.bottom = " << v->getBorder() << ";" << endl;
    _h << "        rborder.right = " << v->getHBorder() << ";" << endl;
    _h << "        MapDialogRect(hDlg, &rborder);" << endl;

    _h << "        ContainerControlRuntime *obj" << (void*)v << " = ";
    _h << "new ContainerControlRuntime(obj" << (void*)v->getControl();
    _h << ", rborder.right, rborder.bottom, ";

    if (v->getWidth() > 0)
    {
       _h << "rect.right, ";
    }
    else
    {
       _h << v->getWidth() << ", ";
    }

    if (v->getHeight() > 0)
    {
       _h << "rect.bottom, ";
    }
    else
    {
       _h << v->getHeight() << ", ";
    }

    switch (v->getWAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ", ";
    switch (v->getHAlign())
    {
       case Sizeable::TOPLEFT:
          _h << "TOPLEFT";
          break;
       case Sizeable::BOTTOMRIGHT:
          _h << "BOTTOMRIGHT";
          break;
       case Sizeable::CENTER:
          _h << "CENTER";
          break;
    }
    _h << ");" << endl;
    _h << "        obj" << (void*)v << "->addChild(obj" << (void*)v->getChild();
    _h << ");" << endl;
}

static const char* _rcsid_Parser __attribute__((unused)) = "$Id$";
