
/**
 * @file Parser.h
 * @brief Declaration of class Parser
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#ifndef _PARSER_H
#define _PARSER_H

#include <iostream>
#include <list>
#include <string>

#include "SAXParser.h"
#include "Visitor.h"

class IDGen;
class Icon;
class MenuItem;
class DialogRC;
class Container;
class ControlRC;
class ContainerControlRC;

#define TYPE_ID        0
#define TYPE_ICON      1
#define TYPE_RAW       2
#define TYPE_MENU      3
#define TYPE_DIALOG    4
#define TYPE_CONTAINER 5
#define TYPE_CONTROL   6
#define TYPE_INT       7

typedef struct
{
      int type;
      union
      {
            IDGen *id;
            Icon *icn;
            std::string *raw;
            MenuItem *menu;
            DialogRC *dlg;
            Container *ctn;
            ControlRC *ctrl;
            int ival;
      } value;
} U_STACK;

class Parser : public SAXParser, public Visitor
{
  public:
    Parser(std::ostream&,     // .RC
           std::ostream&,     // .H
           bool);
    virtual ~Parser();

    void startElement(const std::string&,
                      const std::map<std::string, std::string>&);
    void endElement(const std::string&);
    void text(const std::string&);

    void parse(const std::string&);

    void visit(SimpleContainer*);
    void visit(Box*);
    void visit(ContainerControlRC*);
    void visit(ControlRC*);
    void visit(DialogRC*);

  private:
    std::ostream& _rc;
    std::ostream& _h;
    bool _genh;

    int _state;

    std::list<int> _stack;
    std::list<U_STACK> _ostack;
    std::string _text;
};

#endif /* _PARSER_H */
