
/**
 * @file SAXParser.cpp
 * @brief Implementation of class SAXParser
 */

/**********************************************************************

  Created: 30 oct. 2002

    Copyright (C) 2002 laheurte@LAHEURTE

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include "SAXParser.h"

#include <libxml/parser.h>
#include <libxml/parserInternals.h>

using namespace std;

static void startElementCB(void *ud, const char *n, const char** a)
{
    map<string, string> attrs;
    if (a)
    {
       int k = 0;
       while (a[k])
       {
          attrs.insert(make_pair(a[k], a[k+1]));
          k += 2;
       }
    }

    ((SAXParser*)ud)->startElement(n, attrs);
}

static void endElementCB(void *ud, const char *name)
{
    ((SAXParser*)ud)->endElement(name);
}

static void textCB(void *ud, const char *t, int l)
{
    ((SAXParser*)ud)->text(string(t, t+l));
}

void SAXParser::parse(const std::string& fn)
{
    // bug in libmlx2-2.4.26.win32 !!
    xmlSAXHandler *hdr = (xmlSAXHandler*)malloc(2 * sizeof(xmlSAXHandler));
    memset(hdr, 0, 2 * sizeof(*hdr));
    hdr->startElement = (startElementSAXFunc)&startElementCB;
    hdr->endElement = (endElementSAXFunc)&endElementCB;
    hdr->characters = (charactersSAXFunc)&textCB;

    xmlParserCtxtPtr ctxt;
    ctxt = xmlCreateFileParserCtxt(fn.c_str());
    ctxt->sax = hdr;
    ctxt->userData = (void*)this;
    xmlParseDocument(ctxt);
    xmlFreeParserCtxt(ctxt);
    free(hdr);
}

static const char* _rcsid_SAXParser __attribute__((unused)) = "$Id$";
