
/**
 * @file Sizeable.cpp
 * @brief Implementation of class Sizeable
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include "Sizeable.h"

Sizeable::Sizeable(int w,
                   int h,
                   teAlignment wa,
                   teAlignment ha)
    : _w(w), _h(h), _wa(wa), _ha(ha), _rh(-1)
{
}

unsigned int Sizeable::getMinWidth()
{
    return (_w < 0) ? 14 : (unsigned int)_w;
}

unsigned int Sizeable::getMinHeight()
{
    return (_h < 0) ? 14 : (unsigned int)_h;
}

unsigned int Sizeable::getMinDim(teOrientation o)
{
    return (o == HORIZONTAL) ? getMinWidth() : getMinHeight();
}

Sizeable::teAlignment Sizeable::getWAlign() const
{
    return _wa;
}

Sizeable::teAlignment Sizeable::getHAlign() const
{
    return _ha;
}

Sizeable::teAlignment Sizeable::getAlign(teOrientation o) const
{
    return (o == HORIZONTAL) ? getWAlign() : getHAlign();
}

int Sizeable::getWidth() const
{
    return _w;
}

int Sizeable::getHeight() const
{
    return _h;
}

int Sizeable::getDim(teOrientation o) const
{
    return (o == HORIZONTAL) ? getWidth() : getHeight();
}

void Sizeable::_compute(teOrientation o, int& pos, int& dim)
{
    if (getDim(o) > 0)
    {
       int rdim = getDim(o);
       switch (getAlign(o))
       {
          case TOPLEFT:
             // nothing
             break;
          case BOTTOMRIGHT:
             pos = pos + dim - rdim;
             break;
          case CENTER:
             pos = pos + (dim - rdim)/2;
             break;
       }
       dim = rdim;
    }
}

void Sizeable::setRH(int r)
{
    _rh = r;
}

static const char* _rcsid_Sizeable __attribute__((unused)) = "$Id$";
