
/**
 * @file xmlrc.cpp
 */

/**********************************************************************

  Created: 03 nov. 2002

    Copyright (C) 2002 @SAURON

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA

**********************************************************************/
// $Id$

#include <string>
#include <fstream>

#include "Parser.h"
#include "IDGen.h"
#include "xmlrc.h"

using namespace std;

static void usage(void)
{
    cout << "Usage: xmlrc [options] file[.xml]" << endl;
    cout << "Options are:" << endl;
    cout << "  -noc         no not generate C++ code" << endl;
    cout << "  -p <prefix>  use <prefix> for generated files" <<endl;
    cout << "  -i <start>   where to start auto-generated IDs" << endl;
    exit(1);
}

int main(int argc, char *argv[])
{
    bool genC = true;
    string prefix;

    // first, default prefix
    if (argc < 2) usage();
    string file = argv[argc-1];
    argc--;

    string::reverse_iterator k = file.rbegin();
    int s = 0;
    while ((s != 4) && (s != 5))
    {
       switch (s)
       {
          case 0:
             switch (*k)
             {
                case 'l':
                case 'L':
                   s = 1;
                   break;
                default:
                   s = 4;
                   break;
             }
             break;
          case 1:
             switch (*k)
             {
                case 'm':
                case 'M':
                   s = 2;
                   break;
                default:
                   s = 4;
                   break;
             }
             break;
          case 2:
             switch (*k)
             {
                case 'x':
                case 'X':
                   s = 3;
                   break;
                default:
                   s = 4;
                   break;
             }
             break;
          case 3:
             switch (*k)
             {
                case '.':
                   s = 5;
                   break;
                default:
                   s = 4;
                   break;
             }
             break;
       }
       ++k;
    }

    if (s == 5)
    {
       prefix = string(k, file.rend());
       prefix = string(prefix.rbegin(), prefix.rend());
    }
    else
    {
       prefix = file;
       file += ".xml";
    }

    // les options
    for (int j = 1; j<argc; ++j)
    {
       if (!strcmp(argv[j], "-p"))
       {
          if (j+1 >= argc) usage();
          prefix = argv[j+1];
          j += 2;
       }
       else if (!strcmp(argv[j], "-noc"))
       {
          genC = false;
          ++j;
       }
       else if (!strcmp(argv[j], "-i"))
       {
          if (j+1 >= argc) usage();
          IDGen::setStartID(atoi(argv[j+1]));
          j += 2;
       }
       else
       {
          usage();
       }
    }

    ofstream rcfile((prefix + ".rc").c_str());

    rcfile <<
       "/*" << endl <<
       "     " << prefix << ".rc generated by Xmlrc V" VERSTRING << endl <<
       "     Do not edit by hand!" << endl <<
       "*/" << endl << endl;

    rcfile << "#include <windows.h>" << endl;
    rcfile << "#include <winver.h>" << endl;
    rcfile << endl;

    if (genC)
    {
       rcfile << "#include \"" << prefix << ".h\"" << endl;
       rcfile << endl;
    }

    if (genC)
    {
       ofstream hfile((prefix + ".h").c_str());

       hfile <<
          "/*" << endl <<
          "     " << prefix << ".h generated by Xmlrc V" VERSTRING << endl <<
          "     Do not edit by hand!" << endl <<
          "*/" << endl << endl;

       hfile << "#ifndef _" << prefix << "_h_" << endl;
       hfile << "#define _" << prefix << "_h_" << endl;
       hfile << endl;

       hfile << "#include <windows.h>" << endl;
       hfile << endl;

       Parser p(rcfile, hfile, true);
       p.parse(file);

       hfile << endl;
       hfile << "#endif /* _" << prefix << "_h_ */" << endl;
    }
    else
    {
       Parser p(rcfile, cout, false);
       p.parse(file);
    }

    return 0;
}

static const char* _rcsid_xmlrc __attribute__((unused)) = "$Id$";
